import * as THREE from "three";
import { Sky } from "three/examples/jsm/objects/Sky.js";
import { 
alertAsync
,confirmAsync 
,choiceAsync
,promptAsync
} from "./dialogUtil";
import { 
loadModel
,loadModelFromArrayBuffer
,loadPly
,loadPlyFromText
,makePlyGeometryFromText
,faceTo
} from "./modelUtil";


//signaling_url シグナリングサーバのurl
//ローカルPC上だけで動かす場合は変更不要
//もしlan内で動かす場合はシグナリングサーバを動かす端末の
//実際のipを指定する。
//シグナリングサーバをsslモードで動かしている場合はwss://で
export const signaling_url = "ws://127.0.0.1:14514";


/*
ルームの設定はここに書く
geodataを編集して地形を設定する。
コードで地形配置をしたい場合はinit_geodataを編集する。
光源設定を変えたい場合はinit_light_settingsを編集する。
*/

//geodata 部屋に表示するオブジェクト定義の配列
/*
項目定義
type 種類
  plane 色だけ指定する平面
  box 色だけ指定する箱
  glb glbファイルによる3dモデル resourceでパス指定
  ply plyファイルによる3dモデル resourceでパス指定
    plyについてはMagicaVoxelでエクスポートしたもの前提

resource モデルデータ
  type=glb plyの場合のファイルの場所

color 色
  type=boxの場合の箱の色
  16進リテラルで表記 例 0xff0000

px py pz 位置
  オブジェクトの配置位置

sx sy sz 大きさ
  オブジェクトの大きさ
  planeの場合はsx szのみ有効
  boxの場合はサイズそのもの
  glb plyの場合は倍率

animation アニメーション
  glbのみ有効 autoにすると自動再生
  アニメーションありのglbはオブジェクトをobjとするとき
  obj.actions[]にアニメーション名をキーとしてAnimationActionが積まれる
  それに対して play/stopメソッドで操作できる。
  例 obj["Take 001"].play()

collision 物理判定
  wall 侵入禁止
  none 判定なし
  そのほか 移動可能地形（階段とか）

rx ry rz 3dモデルの向き
  type=boxの場合は無効
  ※物理演算ではなく矩形範囲による疑似判定なので0 0 0以外は見た目と異なる場合がある

onSelectDown ロングタップのイベントハンドラ
  オブジェクトをロングタップした場合に呼ばれる
  引数はオブジェクトのインスタンス

onLoad 生成時のイベントハンドラ
  引数はオブジェクトのインスタンス
  

*/


export var geodata = [
  {
    type:"plane",
    name:"ground",
    px:0,
    py:0,
    pz:0,
    sx:50,
    sy:1,
    sz:50,
    color:0x5e5e5e,
    collision:"land",
    onLoad:async (obj)=>{
      const gridHelper = new THREE.GridHelper(50, 50);
      obj.parent.add(gridHelper);
    }
  },
/*
  {
    type:"box",
    px:5,
    py:0.5,
    pz:0,
    sx:1,
    sy:1,
    sz:1,
    color:0xff0000,
    collision:"wall",
    onSelectDown:async ()=>{
      await alertAsync("こいつは障害物です");
    }
  },
  {
    type:"glb",
    px:0,
    py:0,
    pz:-5,
    sx:0.1,
    sy:0.1,
    sz:0.1,
    rx:0,
    ry:0,
    rz:0,
    resource:"asset/model/an_animated_cat.glb",
    color:0xffffff,
    collision:"wall",
    animation:"",
    onSelectDown:async (obj)=>{
      faceTo(obj,window.myplayer,0);
      console.log(obj.actions['Take 001']);
      if(obj.actions['Take 001'].isRunning()){
        await alertAsync("にゃ？");
        obj.actions['Take 001'].stop();
      }else{
        await alertAsync("にゃーん");
        obj.actions['Take 001'].play();
      }
      //window.the_cat = obj;
      
    }
  },
  {
    type:"ply",
    px:0,
    py:0,
    pz:5,
    sx:1,
    sy:1,
    sz:1,
    rx:0,
    ry:0,
    rz:0,
    resource:"asset/model/test.ply",
    color:0xffffff,
    collision:"wall",
    onSelectDown:async (obj)=>{
      if(!obj.tap_count){
        obj.tap_count=1;
      };
      faceTo(obj,window.myplayer,Math.PI/2);

      if(obj.tap_count==1){
        await alertAsync("初めまして");
      }else if(obj.tap_count==2){
        await alertAsync("また会いましたね");
      }else{
        await alertAsync("こんにちは");
      }
      obj.tap_count+=1;
    }
  }
*/
];

//init_geodata 地形定義初期化処理
/*
もしコードで地形データ生成したい場合はここでgeodataに追加する
*/
export const init_geodata = () =>{
  //例 螺旋階段
/*
  const r = 10;
  const cx = 0;
  const cz = 0;
  const cy = 0;
  const step_y = 0.25;
  const step_a = Math.PI/32;
  var angle = 0;
  for(let i=0;i<200;i++){
    let px = cx + Math.cos(angle)*r;
    let py = cy + i * step_y;
    let pz = cz + Math.sin(angle)*r;
    angle += step_a;
    const gd = {
      px:px,
      py:py,
      pz:pz,
      sx:1,
      sy:0.25,
      sz:1,
      type:"box",
      color:0xff00ff,
    };
    geodata.push(gd);
  }
*/

}

//init_light_settings 光源初期化
/*
  光源や空などの定義を変更したい場合はここをいじる
*/
export const init_light_settings = (scene)=>{
  //環境光
  {
    const light = new THREE.AmbientLight(0xffffff, 0.2);
    scene.add(light);
  }
  //直線光
  {
    const light = new THREE.DirectionalLight(0xffffff, 0.8);

    light.position.set(0, 10, -10).normalize();

    light.castShadow = true;
    light.shadow.mapSize.width = 1024;
    light.shadow.mapSize.height = 1024;
    light.shadow.camera.near = 500;
    light.shadow.camera.far = 4000;
    light.shadow.camera.fov = 30;


    scene.add(light);
  }

  //空
  {
    const sky = new Sky();
    sky.name = "sky";
    sky.scale.setScalar(450000);
    scene.add(sky);

    const uniforms = sky.material.uniforms;
    const phi = THREE.MathUtils.degToRad(90 - 30);
    const theta = THREE.MathUtils.degToRad(180);
    const sun = new THREE.Vector3();
    sun.setFromSphericalCoords(1, phi, theta);
    uniforms["sunPosition"].value.copy(sun);
  }

}




